<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Glowbase GmbH <support@glowbase.com>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/*
 * Howto
 * 
 * 1) Specify Logfiles (incl. path) in ext_localconf.php in $TYPO3_CONF_VARS[$_EXTKEY]['log'][<name>].
 *    <name> means a class of 'error', 'mail', 'sql', 'browser', ...
 * 2) Call 'log_do($class, $status, $origin, $message)' to write one entry. 
 *    Every entry will start with: [<status>|<origin>|<date/time>|<IP>|<fe.uid>|<fe.username>|<URL>].
 *    Specify $message string or $message array.
 * 3) For 'sql' and 'mail' log entries, use the wrapper log_sql() or log_mail().   
 */

namespace qfq;

//use qfq;

class Log {
    /**
     * @var array
     */
    private $config;    // filled in __construct, Definition: tx_fr_variables->collectGlobalVariables

    /**
     * @var array   order to compare loglevel
     */
    private $config_level = array('E' => 0, 'W' => 1, 'I' => 2, 'D1' => 3, 'D2' => 4, 'D3' => 5);

    /**
     * @var array Emulate global variable: will be set much earlier in other functions. Will be shown in error messages.
     */
    private $fr_error = array();

    /**
     * Initializes class
     *
     * @param    array $tmp_config :    Part of array 'fr_array': 'global.'
     *
     * @throws \Exception
     */
    public function __construct(array $tmp_config) {
//        global $TYPO3_CONF_VARS;
//        $this->prepare_log_file_dir($TYPO3_CONF_VARS[EXTKEY]['log']['sql']);
//        $this->prepare_log_file_dir($TYPO3_CONF_VARS[EXTKEY]['log']['mail']);
//        $this->prepare_log_file_dir($TYPO3_CONF_VARS[EXTKEY]['log']['error']);
//        $this->prepare_log_file_dir($TYPO3_CONF_VARS[EXTKEY]['log']['browser']);

        $this->config = $tmp_config;

    } // __construct()

    /**
     * Set Array fr_error: setter function to set most recent values.
     * Will be shown in log messages.
     *
     * @param    array $fr_error :   uid, pid, row, column_idx, full_level
     *
     * @return    void
     */
    public function set_fr_error(array $fr_error) {
        $this->fr_error = $fr_error;
    }

    /**
     * log_sql: wrapper for log_do() for sql logging
     *
     * @param    string $origin :    Sender of the logmessage. F.e.: 'form', 'save', 'report', 'extjs', ...
     * @param    string $msg : Only filled if there was an error
     * @param    int $affected_rows : Number of rows inserted, updated or deleted.
     * @param    int $new_id : last_insert_id()
     * @param    string $sql : SQL statement fired
     *
     * @return    void
     */
    public function log_sql($origin, $msg, $affected_rows, $new_id, $sql) {

        if ($msg) {
            $status = 'E';
        } else {
            $status = 'I';
            $msg = 'OK';
        }

        $text = "[$msg]";
        $text .= "[$affected_rows]";
        $text .= "[$new_id]";
        $text .= "[$sql]";

        $this->log_do('sql', $status, $origin, $text);
    } // prepare_log_file_dir()

    /**
     * log_do: format log entries, build fix part, append dynamic part, write.
     *
     * @param    string $class :        'error', 'mail', 'sql', 'browser'. New classes has to be defined in ext_localconf.php
     * @param    string $status :    'E' (0:error), 'W' (1:warning), 'I' (2:information), 'D1' (3:debug verbose), 'D2' (4:debug very verbose), 'D3' (5:debug very very verbose)
     * @param    string $orign :        Sender of the logmessage. F.e.: 'form', 'save', 'report', 'extjs', ...
     * @param  string /array        $message:    skalar or one dimensal array with logmessage(s).
     * @throws CodeReportException
     * @throws SyntaxReportException
     */
    public function log_do($class, $status, $origin, $message) {
        global $TYPO3_CONF_VARS;
        $tmp = '';

        // Check if loglevel should be respected and if 'yes' if loglevel is greater than this message: break
        if (($class == 'error' || $class == 'browser') && $this->config_level[$status] > $TYPO3_CONF_VARS[EXTKEY]['log']['level'])
            return;

        // Get filename
        $filename = $TYPO3_CONF_VARS[EXTKEY]['log'][$class];
        if (!$filename)
            throw new SyntaxReportException ("Missing logfile definition: Undefind TYPO3_CONF_VARS[" . EXTKEY . "]['log']['$class']", "", __FILE__, __LINE__);

        if ($this->fr_error['pid']) $tmp = '|' . $this->fr_error['pid'];
        if ($this->fr_error['uid']) $tmp .= '|' . $this->fr_error['uid'];
        if ($this->fr_error['full_level']) $tmp .= '|' . $this->fr_error['full_level'];

        // F.e.: [I|Form|2012.01.31-19:59:33|IP|fe.uid|fe.username|URL]
        $text = '[' . $status . '|' . $origin . '|' . date('Y.m.d-H:i:s') . '|' . $this->config["REMOTE_ADDR"] . '|' . $this->config["fe_user_uid"] . '|' . $this->config["fe_user"] . $tmp . '|' . $this->config["url"] . ']';

        // If 'message' is an array, wrap every element (but not the last) with '[', ']'.
        if ($message && is_array($message)) {
            $last = array_pop($message);
            if ($last && $message) {
                $text .= '[' . implode('][', $message) . ']';
            }
            $message = $last;
        }
        $text .= $message;

        // Write whole entry
        $this->log_write($filename, $text);
    } // log_do()

    /**
     *    Logs every email (failed or successfull)
     *
     * @param    string $orign :        Sender of the logmessage. F.e.: 'form', 'save', 'report', 'extjs', ...
     * @param    string $status :    'E' (0:error), 'W' (1:warning), 'I' (2:information), 'D1' (3:debug verbose), 'D2' (4:debug very verbose), 'D3' (5:debug very very verbose)
     * @param    string $msg :        Message
     * @param    array $mailarr :    Details of the mail
     *
     * $mailarr['sender']      Absender
     * $mailarr['receiver']    Empfaenger, mehrere mit Komma getrennt
     * $mailarr['subject']     Betreff
     * $mailarr['body']        Mailinhalt
     * $mailarr['src']         level oder form_element.id um herauszufinden wer die Mail gesendet hat
     *
     * @param    string $mailarr : data to log
     * @return    The content that is displayed on the website
     */

    public function log_mail($origin, $status, $msg, $mailarr) {

        $text = '[' . $msg . "]";
        $text .= '[' . $mailarr["sender"] . ']';
        $text .= '[' . $mailarr["receiver"] . ']';
        $text .= '[' . $mailarr["subject"] . ']';
        $text .= '[' . $mailarr["body"] . ']';

        $this->log_do('mail', $status, $origin, $text);

    } // log_sql()

    /**
     * Check if path of logfile exists. No: create it.
     * Check if path of logfile is writeable.
     *
     * @param string $filename Name of logfile
     * @throws \Exception
     */
    private function prepare_log_file_dir($filename) {
        $this->logFile = $filename;

        // extract optional path component
        $path = dirname($filename);

        // If there is a path: check if directories really exist and are writeable.
        if ($path) {
            if (file_exists($path)) {
                // Directory already exists: check if it is writeable.
                if (!is_writable($path)) throw new \Exception ("Failed: directory '$path' of Logfile '$filename' is not writeable.");
            } else {
                // Directoy doesn't exist: create it.
                if (!mkdir($path, 0775, true))
                    throw new \Exception ("Failed to create directory '$path' for Logfile '$filename'");
            }
        }
    } // log_mail()

}
