<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/15/16
 * Time: 8:24 AM
 */

namespace qfq;

require_once(__DIR__ . '/AbstractDatabaseTest.php');
require_once(__DIR__ . '/../../qfq/Delete.php');

class DeleteTest extends \AbstractDatabaseTest {

    /**
     * @expectedException \qfq\CodeException
     */
    public function testProcessException() {
        $msg = array();

        $delete = new Delete(true);

        // empty 'form' not allowed
        $delete->process(array(), 123, $msg);
    }

    /**
     * @expectedException \qfq\CodeException
     */
    public function testProcessException1() {
        $msg = array();

        $delete = new Delete(true);

        // 'form' with empty tablename not allowed
        $form[F_TABLE_NAME] = '';
        $delete->process($form, 123, $msg);
    }

    /**
     * @expectedException \qfq\CodeException
     */
    public function testProcessException2() {
        $msg = array();

        $delete = new Delete(true);

        // empty record id not allowed
        $form[F_TABLE_NAME] = 'Person';
        $delete->process($form, '', $msg);
    }

    /**
     * @expectedException \qfq\CodeException
     */
    public function testProcessException3() {
        $msg = array();

        $delete = new Delete(true);
        $form[F_TABLE_NAME] = 'Person';

        // record id = 0 not allowed
        $delete->process($form, 0, $msg);
    }

    /**
     * @expectedException \qfq\CodeException
     */
    public function testProcessException4() {
        $msg = array();

        $delete = new Delete(true);

        // unknown table not allowed
        $form[F_TABLE_NAME] = 'UnknownTable';
        $delete->process($form, 0, $msg);
    }

    /**
     */
    public function testProcessRecordNotFound() {
        $msg = array();

        $delete = new Delete(true);

        // unknown table not allowed
        $form[F_TABLE_NAME] = 'Person';
        $rc = $delete->process($form, 100, $msg);

        $this->assertEquals(false, $rc);

        $expect = ['content' => "Record 100 not found in table 'Person'.", 'errorCode' => 1065];
        $this->assertEquals($expect, $msg);
    }

    /**
     */
    public function testProcess() {
        $msg = array();

        $delete = new Delete(true);

        // unknown table not allowed
        $form[F_TABLE_NAME] = 'Person';
        $rc = $delete->process($form, 1, $msg);

        $this->assertEquals(true, $rc);

        $this->assertEquals(array(), $msg);

        $count = $this->db->sql('SELECT COUNT(id) FROM Person', ROW_IMPLODE_ALL);
        $this->assertEquals('1', $count);

    }


    protected function setUp() {

        $this->store = Store::getInstance('form=TestFormName', true);
        parent::setUp();

        $this->store->setVar('form', 'TestFormName', STORE_TYPO3);
        $this->store->setVar(SYSTEM_SITE_PATH, '/tmp', STORE_SYSTEM, true);

        $this->executeSQLFile(__DIR__ . '/fixtures/Generic.sql', true);
    }

}
